@@ -115,7 +115,8 @@ GLFS_MSGID(PS,
         PS_MSG_SEEK_INFO,
         PS_MSG_COMPOUND_INFO,
         PS_MSG_CLIENT_OPVERSION_GET_FAILED,
-        PS_MSG_CHILD_STATUS_FAILED
+        PS_MSG_CHILD_STATUS_FAILED,
+        PS_MSG_PUT_INFO
 );
 
 #endif /* !_PS_MESSAGES_H__ */
